/* @(#) sm_api.h 1.62 3/26/98 12:12:35 */

//////////////////////////////////////////////////////////////////////////
//
// FILE:  sm_api.h
// DESCRIPTION:
//    Includes class definitions, compiler defines, and prototypes that
// the application might use.  This header is included through sm_internal.h
// by hilevel, lolevel, and asn1 code.  It is included indirectly by the
// application through the CTI Library Headers (e.g. sm_fort.h).
//
//    The classes in this header consist of the following "categories":
// 1. General API Classes:  Classes that the application must interact
//    with such as CSMIME and CSInst.
// 2. High Level Message Classes:  Classes that most applications will
//    interact with to work with CSM Objects such as CSM_SignMsg,
//    CSM_VerifyMsg, CSM_EncryptMsg, CSM_DecryptMsg, and CSM_ContentInfoMsg.
// 3. Low Level message Classes:  Classes that a few applications may
//    choose to interact with instead of those mentioned in 2 above
//    because they want direct access to the SNACC classes.  These
//    classes include CSM_SignData, CSM_VerifyData, CSM_EncryptData, etc...
// 4. Low Level Component Classes:  low level classes that the application
//    may use in conjunction with the high level classes such as 
//    CSM_Buffer, CSM_OID, and the attribute classes (e.g. CSM_ContentHints)
// 5. Classes that the Application will not normally use but are defined
//    here for use by the SFL such as CSM_TokenInterface and 
//    CSM_BaseTokenInterface.
//
//////////////////////////////////////////////////////////////////////////
#ifndef _SM_API_H_
#define _SM_API_H_



// SNACC header
//
#include "asn-incl.h"

// SNACC Generated headers
//
#include "sm_vdatypes.h"
#include "sm_x501ud.h"
#include "sm_x411ub.h"
#include "sm_x411mtsas.h"
#include "sm_x501if.h"
#include "sm_x520sa.h"
#include "sm_x509cmn.h"
#include "sm_x509af.h"
#include "sm_x509ce.h"
#include "sm_cms.h"
#include "sm_ess.h"
//
// End of SNACC generated headers

#include "sm_error.h"   // error classes and macros

///////////////////////
// DEFINED CONSTANTS //
///////////////////////

#define SM_ISS_AND_SERIAL_NO     0   /* Recipient Identifier union choices */
#define SM_RKEY_ID               1
#define SM_MLKEY_ID              2
#define SM_FirstTierRecipients   2
#define SM_AllReceipts           1
#define SM_NoReceipt             0
#define SM_receiptList           -1

// SM_ENV_DATA_VERSION is used if there is an originator info or if
// there is a recipient info with version SM_ENV_DATA_VERSION in the
// envelopedData
#define SM_ENV_DATA_VERSION      2
// otherwise, SM_ENV_DATA_PREV_VERSION is used
#define SM_ENV_DATA_PREV_VERSION 0

// SM_INST_* defines are used to identify instances that 
// the application wants to lock, unlock, or clear
#define SM_INST_APPLICABLE 0x01
#define SM_INST_USE_THIS 0x02
#define SM_INST_ALL 0x04

////////////////////////////////
// Forward Class Declarations //
////////////////////////////////
class CSM_OID;
class CSM_SecCat;
class CSM_Ml;
class CSM_CertificateChoice;
class CSM_UserKeyMaterial;
class CSM_Recipient;
class CSM_CSInst;
class CSM_Alg;
class CSM_MsgCertCrls;
class CSM_MsgSignerInfo;
class CSM_MsgSignerInfos;
class CSM_MsgAttributes;
class CSM_ReceiptRequest;
class CSM_SecLbl;
class CSM_GeneralAsn;
class CSM_ErrorBuf;
class CSM_VerifyMsg;
class CSM_DecryptMsg;
class CSM_DN;
class CSM_Attrib;
class CSM_OriginatorInfo;
class CSM_Content;
class CSM_HashDef;
class MAB_Entrydef;


///////////////////////
// List Declarations //
///////////////////////
typedef CSM_List<CSM_Alg> CSM_AlgLst;
typedef CSM_List<CSM_Attrib> CSM_AttribLst;
typedef CSM_List<CSM_Buffer> CSM_BufferLst;
typedef CSM_List<CSM_CertificateChoice> CSM_CertificateChoiceLst;
typedef CSM_List<CSM_CSInst> CSM_CSInstLst;
typedef CSM_List<CSM_DN> CSM_DNLst;
typedef CSM_List<CSM_HashDef> CSM_HashDefLst;
typedef CSM_List<CSM_Ml> CSM_MlLst;
typedef CSM_List<CSM_MsgSignerInfo> CSM_MsgSignerInfoLst;
typedef CSM_List<CSM_SecCat> CSM_SecCatLst;
typedef CSM_List<CSM_Recipient> CSM_RecipientLst;
typedef CSM_List<CSM_UserKeyMaterial> CSM_UserKeyMaterials;

///////////////////////
// CLASS DEFINITIONS //
///////////////////////

//////////////////////////////////////////////////////////////////////////
// CSMIME contains the CSInst instances as well as overall library
// information (including the error handling buffers).
class CSMIME {
private:
   CSM_ErrorBuf m_ErrorBuf;   // Contains error/debug information on error.
public:
   CSM_CSInstLst *m_pCSInsts; // instance list

   // constructors and destructors
   CSMIME() { m_pCSInsts = NULL; }
   ~CSMIME() { if (m_pCSInsts) delete m_pCSInsts; }

   // UseAll marks every instance's UseThis to true
   void UseAll();

   // The UseAllSigners members assist the application in automatically
   // selecting instances to sign with
   SM_RET_VAL UseAllSigners();
   // The following two UseAllSigners selects singers based on
   // the DS Alg OIDs in the provided certificates
   SM_RET_VAL UseAllSigners(CSM_Buffer *pCert);
   SM_RET_VAL UseAllSigners(CSM_BufferLst *pCerts);

   // Find the first instance in list that contains the following list
   //  of algorithms.  Any of the parameters may be NULL; a instance is
   //  returned only if all specified algorithms are present.
   CSM_CSInst* FindCSInstAlgIds(CSM_Alg *pdigestAlgID, 
         CSM_Alg *pdigestEncryptionAlgID,
         CSM_Alg *pkeyEncryptionAlgID,
         CSM_Alg *pcontentEncryptionAlgID);

   // returns a pointer to the CSM_ErrorBuf class (error information)
   CSM_ErrorBuf* GetErrorBuf() { return &m_ErrorBuf; }   

   //  SetCSInstVerify will assign all appropriate CSInstances that can verify
   //  the signature of an incomming SignedData message.
   SM_RET_VAL SetCSInstVerify(CSM_VerifyMsg *pVerifyMsg);

   //  FindCommonContentOID will determine a common Content algorithm based on
   //  the specified recipient(s).  If more than one recipient is 
   //  specified, then an attempt is made to find a content encryption
   //  algorithm common to each pair-wise key encryption algorithm.
   SM_RET_VAL FindCommonContentOID(CSM_BufferLst *pRecipCerts,
         CSM_OID *pContentEncryptOID);           // May be NULL

   //  SetCSInstDecrypt will assign all appropriate CSInstances that can decrypt
   //  messages from the recipient(s) in CSM_DecryptMsg.  This may entail 
   //  searching among the various CSInstances for one or more of the intended
   //  recipients present in the message (based on the RecipientInfos in the
   //  received EnvelopedData message).  If successful, the 
   //  CSM_DecryptMsg->Decrypt() member function can be called; a CSInstance was
   //  located that matches one of the personalities in the message.
   //  If more than one recipient can be processed by this set of CSInstances, 
   //  this function will choose the first valid recipient in the list; it is
   //  expected that the application will explicitely mark the appropriate
   //  CSInstance if an alternate recipient is required.
   SM_RET_VAL SetCSInstDecrypt(CSM_DecryptMsg *pDecryptMsg);

   // InstanceLock, InstanceUnlock, and the SM_INST_* defines are
   // used to SMTI_Lock and SMTI_Unlock groups of instances based on
   // their applicable flag (APPLICABLE), their UseThis flag (USE_THIS)
   // or a combination thereof...
   void InstanceLock(char chLockFlag);
   void InstanceUnlock(char chLockFlag);

   // ClearFlag clears the specified flags from the instances (APPLICABLE
   // and/or USE_THIS)
   void ClearFlag(char chClearFlag);
};

//////////////////////////////////////////////////////////////////////////
// CSM_TokenInterface is not directly used by the application.
// It is used internally by the SFL to abstractly access crypto services.
class CSM_TokenInterface
{
public:
   // SMTI_Sign acquires the necessary X and parameters as
   // appropriate.  SMTI_Sign digests the data and returns the
   // digest in pDigest.  SMTI_Sign encrypts the digest and returns
   // the encrypted digest in pEncryptedDigest.
   virtual SM_RET_VAL SMTI_Sign(
            CSM_Buffer *pData, // input, data to be signed
            CSM_Buffer *pEncryptedDigest, // signature
            CSM_Buffer *pDigest) =0; // digest output

   // SMTI_Verify accepts the public key of the signer as input
   // and also the data that was signed.  Finally, it accepts
   // the signature (encrypted digest).  It returns
   // SM_NO_ERROR if the signature was verified successfully.
   virtual SM_RET_VAL SMTI_Verify(
            CSM_Buffer *pSignerPublicKey, // input
            CSM_Alg    *pDigestAlg, // input
            CSM_Alg    *pSignatureAlg, // input
            CSM_Buffer *pData, // input
            CSM_Buffer *pSignature) =0; // input

   // SMTI_Encrypt accepts the data to be encrypted.  It returns
   // the encrypted data and the IV.  The class derived from
   // this class must return the MEK for future use with
   // SMTI_GenerateEMEK.  In special cases such as hardware
   // fortezza where the MEK is not available, SMTI_Encrypt
   // may return a special phrase in the buffer to be
   // recognized by whatever CTI's regcognize it.  See the API
   // for a detailed explanation of this.
   virtual SM_RET_VAL SMTI_Encrypt(
            CSM_Buffer *pData, // input (data to be encrypted)
            CSM_Buffer *pEncryptedData, // output
            CSM_Buffer *pParameters, // output, parameters for alg.
            CSM_Buffer *pMEK) =0; // output

   // SMTI_GenerateEMEK accepts the public key (Y) of the recipient
   // as input.  It protects the provided MEK 
   // and returns the result in pEMEK.
   // SMTI_GenerateEMEK also generates a ukm, if applicable...
   virtual SM_RET_VAL SMTI_GenerateEMEK(
            CSM_Buffer *pRecipient, // input, Y of recip
            CSM_Buffer *pParameters, // output, parameters for alg.
            CSM_Buffer *pMEK, // input, MEK or special phrase
            CSM_Buffer *pEMEK, // output, encrypted MEK
            CSM_Buffer *pUKM) =0; // output, ukm, if applicable

   // SMTI_ExtractMEK accepts the public key (Y) of the originator
   // and the encrypted MEK as input.  In RSA's case, pOriginator
   // may be null, otherwise, it uses the Y in conjunction with
   // the recipient's X (available through this class) to generate
   // a TEK that can decrypt the EMEK thus resulting in the MEK.
   // pUKM is TBD
   virtual SM_RET_VAL SMTI_ExtractMEK(
            CSM_Buffer *pOriginator, // input, Y of originator
            CSM_Buffer *pParameters, // input, parameters for alg.
            CSM_Buffer *pEMEK, // input, encrypted MEK
            CSM_Buffer *pUKM, // input
            CSM_Buffer *pMEK) =0; // output

   // SMTI_Decrypt accepts the IV as input.
   // SMTI_Decrypt accepts the encrypted data as input.
   // SMTI_Decrypt uses the MEK extracted by SMTI_ExtractMEK to
   // decrypt the pEncryptedData.  It places the results in pData.
   virtual SM_RET_VAL SMTI_Decrypt(
            CSM_Buffer *pParameters, // input, parameters for alg.
            CSM_Buffer *pEncryptedData, // input (data to be decrypted)
            CSM_Buffer *pMEK, // input (MEK or special phrase)
            CSM_Buffer *pData) =0; // output (decrypted data)

   // SMTI_DigestData digests the data in pData and returns the 
   // result in pDigest
   virtual SM_RET_VAL SMTI_DigestData(
            CSM_Buffer *pData, // input
            CSM_Buffer *pDigest) =0; // output

   // SMTI_Random uses the seed data in pSeed to seed the random generator.
   // depending on the crypto token library implementation, this may be
   // unused.  SMTI_Random fills pRandom with lLength bytes of
   // random data.  
   virtual SM_RET_VAL SMTI_Random(
            CSM_Buffer *pSeed, // input
            CSM_Buffer *pRandom, // input/output
            SM_SIZE_T lLength) =0; // input

   // SMTI_Lock and SMTI_Unlock should be used by the calling hilevel
   // member to lock the crypto resource.  This is particularly applicable
   // to Encrypt/GenerateEMEK and ExtractMEK/Decrypt sequences...
   virtual SM_RET_VAL SMTI_Lock() =0;
   virtual SM_RET_VAL SMTI_Unlock() =0;

   // the following pure virtual members are overriding by co-derived class
   // CSM_BaseTokenInterface
   virtual void BTISetPreferredCSInstAlgs(CSM_OID *oidDigest,
            CSM_OID *oidDigestEncryption,
            CSM_OID *oidKeyEncryption,
            CSM_OID *oidContentEncryption) =0;
   virtual void BTIGetPreferredCSInstAlgs(CSM_OID *&oidDigest,
            CSM_OID *&oidDigestEncryption,
            CSM_OID *&oidKeyEncryption,
            CSM_OID *&oidContentEncryption) =0;
   virtual void BTISetAlgIDs(CSM_AlgLst *pdigestAlgID, 
            CSM_AlgLst *pdigestEncryptionAlgID,
            CSM_AlgLst *pkeyEncryptionAlgID,
            CSM_AlgLst *pcontentEncryptionAlgID) =0;
   virtual void BTIGetAlgIDs(CSM_AlgLst **ppdigestAlgID, 
            CSM_AlgLst **ppdigestEncryptionAlgID,
            CSM_AlgLst **ppkeyEncryptionAlgID,
            CSM_AlgLst **ppcontentEncryptionAlgID) =0;
   virtual bool BTIFindAlgIds(CSM_Alg *pdigestAlgID, 
            CSM_Alg *pdigestEncryptionAlgID,
            CSM_Alg *pkeyEncryptionAlgID,
            CSM_Alg *pcontentEncryptionAlgID) =0;
   virtual CSM_OID* GetPrefDigest() =0;
   virtual CSM_OID* GetPrefDigestEncryption() =0;
   virtual CSM_OID* GetPrefKeyEncryption() =0;
   virtual CSM_OID* GetPrefContentEncryption() =0;

};

//////////////////////////////////////////////////////////////////////////
// CSM_BaseTokenInterface is not directly used by the application.
// CSM_BaseTokenInterface contains base token interface information 
// common to all classes derived from the CSM_TokenInterface
class CSM_BaseTokenInterface : public CSM_TokenInterface
{
private:
   CSM_ErrorBuf *m_pErrorBuf; // usually points to CSMIME error buf

   CSM_AlgLst *m_pDigestAlgorithms;
   CSM_AlgLst *m_pDigestEncryptionAlgorithms;
   CSM_AlgLst *m_pKeyEncryptionAlgorithms;
   CSM_AlgLst *m_pContentEncryptionAlgorithms;
   // private preferred algorithms for encryption and signing operations, if
   //  there are multiple choices.  These can be overriden by the application.
   //  They should be set by the logon process to a default algorithm from the
   //  above "SM_AlgList" parameters.
   CSM_OID m_digestOID;
   CSM_OID m_digestEncryptionOID;
   CSM_OID m_keyEncryptionOID;
   CSM_OID m_contentEncryptionOID;

public:
   CSM_BaseTokenInterface() {
      m_pErrorBuf = NULL;
      m_pDigestAlgorithms = m_pDigestEncryptionAlgorithms =
            m_pKeyEncryptionAlgorithms = 
            m_pContentEncryptionAlgorithms = NULL; }

   // BTISetPreferrredCSInstAlgs will set the preferred encryption and signing
   //  algorithms by CSM_OID.  They must be part of the listed algorithms supported
   //  by this session or the member function will return an error (non-zero).
   //  NULL parameter are acceptable to allow the default session algorithms.
   void BTISetPreferredCSInstAlgs(CSM_OID *oidDigest,
            CSM_OID *oidDigestEncryption,
            CSM_OID *oidKeyEncryption,
            CSM_OID *oidContentEncryption);
   void BTIGetPreferredCSInstAlgs(CSM_OID *&oidDigest,
            CSM_OID *&oidDigestEncryption,
            CSM_OID *&oidKeyEncryption,
            CSM_OID *&oidContentEncryption);
   CSM_OID* GetPrefDigest() { return &m_digestOID; }
   CSM_OID* GetPrefDigestEncryption() { return &m_digestEncryptionOID; }
   CSM_OID* GetPrefKeyEncryption() { return &m_keyEncryptionOID; }
   CSM_OID* GetPrefContentEncryption() { return &m_contentEncryptionOID; }
   void ClearDigestEncryptionCapability();
   void ClearKeyEncryptionCapability();
   void BTISetAlgIDs(CSM_AlgLst *pdigestAlgID, 
            CSM_AlgLst *pdigestEncryptionAlgID,
            CSM_AlgLst *pkeyEncryptionAlgID,
            CSM_AlgLst *pcontentEncryptionAlgID);
   void BTIGetAlgIDs(CSM_AlgLst **ppdigestAlgID, 
            CSM_AlgLst **ppdigestEncryptionAlgID,
            CSM_AlgLst **ppkeyEncryptionAlgID,
            CSM_AlgLst **ppcontentEncryptionAlgID);
   // BTIFindAlgIds will return true if all of the specified Algorithm IDs
   //  are located within this session instance.  Parameters may be NULL.
   bool BTIFindAlgIds(CSM_Alg *pdigestAlgID, 
            CSM_Alg *pdigestEncryptionAlgID,
            CSM_Alg *pkeyEncryptionAlgID,
            CSM_Alg *pcontentEncryptionAlgID);
};

/**************************************************************************/
/* DN structure                                                           */
/**************************************************************************/
class CSM_DN
{
private:
   char *m_pszDN;               /* DN String, OPTIONAL. */
   CSM_Buffer *m_pEncodedDN;    /* ASN.1 encoded DN, OPTIONAL. */
   Name *m_pSNACCName;

public:
   // CONSTRUCTORS
   // use this constructor to create a complete empty DN structure
   CSM_DN();
   // use this constructor to make a copy of the provided DN structure
   // and put it into this structure
   CSM_DN(CSM_Buffer *PCSMB);
   CSM_DN(Name &issuer);

   char *GetDNString();
   void SetEncodedDN(CSM_Buffer &EncCSMB);
   CSM_Buffer *GetEncodedDN();
   void Decode();

   // COMPARISON OPERATORS
   bool operator == (CSM_DN &dn) 
   {
       CSM_Buffer *pTmpDn=dn.GetEncodedDN();
       if (GetEncodedDN() != NULL && pTmpDn)
         return(*m_pEncodedDN == *dn.GetEncodedDN());
       else 
         return(false);
   }
  
   // returns a class pointer
   CSM_DN &operator = (Name &SNACCDN);

   // returns m_pSNACCName
   operator Name* (); 
};

//////////////////////////////////////////////////////////////////////////
// IssuerAndSerialNumber
class CSM_IssuerAndSerialNumber

{
private:
   CSM_DN *m_pIssuer;        // Certificate Issuer Dn
   CSM_Buffer *m_pSerialNo;  // Certificate Serial Number
   IssuerAndSerialNumber *m_pSNACCIssSn;

public:
   // CONSTRUCTORS
   // use this constructor to create a complete empty structure
   CSM_IssuerAndSerialNumber();
   // use this constructor to make a copy of the provided structure
   // and put it into this structure
   CSM_IssuerAndSerialNumber(CSM_DN &dn, CSM_Buffer &serial);
   // use this constructor to get the issuer and serial number
   // from a given certificate
   CSM_IssuerAndSerialNumber(CSM_Buffer *SNACCCertBuf);
   CSM_IssuerAndSerialNumber(Certificate &SNACCCert);
   CSM_IssuerAndSerialNumber(IssuerAndSerialNumber &SNACCIssSn);

   void Set(Certificate &SNACCCert);
   // returns a pointer to the CSM_DN class  
   CSM_DN* GetIssuer(); 
 
   // returns a pointer to the CSM_Buffer class  
   CSM_Buffer* GetSerialNo();  

   // set m_pIssuer private variable with a CSM_DN issuer
   void SetIssuer(CSM_DN &Issuer);    

   // set m_pIssuer private variable with a SNACC issuer
   void SetIssuer(Name &SnaccIssuer);   

   // set m_pSerialNo private variable
   void SetSerialNo(CSM_Buffer &SerialNo);

   // COMPARISON OPERATORS
   bool operator == (CSM_IssuerAndSerialNumber &issuer);

   CSM_IssuerAndSerialNumber &operator = (IssuerAndSerialNumber &SNACCISN);

   operator IssuerAndSerialNumber* (); 
};

//////////////////////////////////////////////////////////////////////////
// CSM_CSInst is a "crypto service instance" or the class that provides
// access to the crypto services for an application and the SFL.
class CSM_CSInst
{
private:
   CSM_TokenInterface *m_pTokenInterface; 
   char *m_pszCSInstID;
   CSM_CertificateChoiceLst *m_pCertificates;
   CSM_BufferLst *m_pCRLs;
   CSM_IssuerAndSerialNumber *m_pIssuerAndSerialNumber;
   CSM_DN *m_pSubjectDN;
   bool m_UseThis;
   bool m_Applicable;

public:
   CSM_ErrorBuf *m_pErrorBuf; // usually points to CSMIME error buf
   CSM_CSInst();
   ~CSM_CSInst();

   // SetTokenInterface and GetTokenInterface are not used by the
   // application under normal circumstances.  They are used by the SFL
   // and the CTI library
   void SetTokenInterface(CSM_TokenInterface *pTokenInterface);
   CSM_TokenInterface *GetTokenInterface() { return m_pTokenInterface; }

   // The application can use GetID and SetID to specify an identification
   // null terminated string for each instance
   // a default one will be generated on construction
   char* GetID() { return m_pszCSInstID; }
   void SetID(char *pszID);

   // The app can use SetUseThis to indicate that this instance should be
   // used for an origination operation
   void SetUseThis() { m_UseThis = true; }
   // sets the private UseThis flag based on the value of flag
   void SetUseThis(bool flag) { m_UseThis = flag; }

   // indicates if this instance is marked UseThis
   bool IsThisUsed() { return m_UseThis; }

   // Indicates if the current instance can be used to verify or decrypt
   // a message that has been "Pre-processed"
   bool IsApplicable() { return m_Applicable; }
   // sets the private applicable flag, used only for the
   //  case where there were no sessions logged on when
   //  SM_PreProc() was called and the application needs
   //  to choose which sessions to use for verification or
   //  decryption.
   void SetApplicable() { m_Applicable = true; }
   // sets the private applicable flag based on the value of flag
   void SetApplicable(bool flag) { m_Applicable = flag; }

   // The following members can be used to set and get ASN.1 objects from this
   // instance
   void SetCertificates(CSM_BufferLst *pCertificates);
   void SetCertificates(CSM_CertificateChoiceLst *pCertificates);
   CSM_CertificateChoiceLst *GetCertificates() { return m_pCertificates; }
   void SetCRLs(CSM_BufferLst *pCRLs);
   CSM_BufferLst *GetCRLs() { return m_pCRLs; }

   // work with the IssuerAndSerialNumber for an instance
   void SetIssuerAndSerialNumber(CSM_IssuerAndSerialNumber 
         *pIssuerAndSerialNumber);
   CSM_IssuerAndSerialNumber *GetIssuerAndSerialNumber();

   // the following 5 methods call the corresponding BTI*
   // method in the m_pTokenInterface
   void SetPreferredCSInstAlgs(CSM_OID *oidDigest,
            CSM_OID *oidDigestEncryption,
            CSM_OID *oidKeyEncryption,
            CSM_OID *oidContentEncryption);
   void GetPreferredCSInstAlgs(CSM_OID *oidDigest,
            CSM_OID *oidDigestEncryption,
            CSM_OID *oidKeyEncryption,
            CSM_OID *oidContentEncryption);
   void SetAlgIDs(CSM_AlgLst *pdigestAlgID, 
            CSM_AlgLst *pdigestEncryptionAlgID,
            CSM_AlgLst *pkeyEncryptionAlgID,
            CSM_AlgLst *pcontentEncryptionAlgID);
   void GetAlgIDs(CSM_AlgLst *&pdigestAlgID, 
            CSM_AlgLst *&pdigestEncryptionAlgID,
            CSM_AlgLst *&pkeyEncryptionAlgID,
            CSM_AlgLst *&pcontentEncryptionAlgID);
   bool FindAlgIds(CSM_Alg *pdigestAlgID, 
            CSM_Alg *pdigestEncryptionAlgID,
            CSM_Alg *pkeyEncryptionAlgID,
            CSM_Alg *pcontentEncryptionAlgID);
   CSM_DN *GetSubjectDN();
};

//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
// SNACC LOW-LEVEL API CLASSES for SFL OPERATIONS
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////
// The application could use CSM_VerifyData if it wants to directly
// manipulate the SignedData snacc class...
class CSM_VerifyData
{
private:
   Receipt m_SnaccSignedReceipt;
   CSM_Buffer *m_pSignedReceipt;

public:
   SignedData *m_pSnaccSignedData; // SNACC generated class

   CSM_VerifyData() { m_pSnaccSignedData=NULL; }
   CSM_VerifyData(CSM_Buffer *pBlob) { m_pSnaccSignedData=NULL; PreProc(pBlob); }
   CSM_VerifyData(CSMIME *pCSMIME, CSM_Buffer *pBlob) 
   { m_pSnaccSignedData=NULL; PreProc(pCSMIME, pBlob); }

   // If the CSMIME parameter is specified, then all logon instances that can
   //  verify the message's signature will be marked with "UseThis" flag.
   //  If the CSMIME parameter is not specified, then the message is simply
   //  decoded into the SNACC structure.
   SM_RET_VAL PreProc(CSMIME *pCSMIME);
   SM_RET_VAL PreProc(CSM_Buffer *pEncodedBlob); // IN, Resulting SignedData/ContentInfo.
   SM_RET_VAL PreProc(CSMIME *pCSMIME,  // IN,logged-on Instance list
        CSM_Buffer *pEncodedBlob);     // IN, Resulting SignedData/ContentInfo.
   CSM_Buffer *ExtractSignerPublicKey(
        IssuerAndSerialNumber *issuerAndSerialNumber, // IN, certificate ID
        CSM_MsgCertCrls *pMsgCertCrls);    // IN,Originator(s) certs+++
   CSM_CSInst *GetSignerInst(CSMIME *pCSMIME,   // IN, list of logons
        SignerInfo *pSI);  // IN, specific SignerInfo
//
  SM_RET_VAL Verify(CSMIME *pCSMIME,   // IN,logged-on Instance list
      CSM_Buffer *pEncodedBlob,      // IN, SignedData message
      CSM_MsgCertCrls *pMsgCertCrls, // IN,Originator(s) certs+++
      CSM_Buffer *pSignedReceipt);   // OUT,returned receipt (if requested)
   SM_RET_VAL CSM_VerifyData::Verify(
    CSMIME *pCSMIME,               // IN, list of logons
    SignerInfo *pSI,               // IN, specific SignerInfo to process.
    CSM_MsgCertCrls *pMsgCertCrls, // IN,Originator(s) certs+++
    CSM_MsgAttributes *pAuthAttrs, // IN, optional signed attributes
    CSM_Buffer *pSignedReceipt);   // OUT,returned receipt (if requested)
   SM_RET_VAL Verify(
      CSM_CSInst *pCSInst,           // IN,logged-on Instance
      CSM_MsgCertCrls *pMsgCertCrls, // IN,Originator(s) certs+++
      CSM_Buffer *pSignedReceipt);   // OUT,returned receipt (if requested)
   SM_RET_VAL Verify(CSMIME *pCSMIME,   // IN,logged-on Instance list
         CSM_MsgCertCrls *pMsgCertCrls, // IN,Originator(s) certs+++
         CSM_Buffer *pSignedReceipt);  // OUT,returned receipt (if requested)
};

//////////////////////////////////////////////////////////////////////////
// The application could use CSM_VerifyReceiptData if it wants direct
// access to the snacc class inherited through CSM_VerifyData.  This class
// would be used to verify a ContentInfo/SignedData/Receipt.
class CSM_VerifyReceiptData : public CSM_VerifyData
{
private:
   // contains encoded msg for rcpt processing
   CSM_Content *m_pOriginalMessage;

public:
   // Set original message for SM_ValReceipt() processing.
   void SetOriginalMessage(CSM_Content  *pOrigMsg);

   SM_RET_VAL Verify(CSMIME *pCSMIME,    // IN,logged-on Instance list
         CSM_MsgCertCrls *pMsgCertCrls); // IN,Originator(s) certs+++
};

//////////////////////////////////////////////////////////////////////////
// The application could use CSM_SignData if it wants to directly
// manipulate the SignedData snacc class...
class CSM_SignData
{
public:
      SignedData m_SnaccSignedData; // SNACC generated class

      SM_RET_VAL Sign(CSMIME *pCSMIME,     // IN,logged-on Instance list
            CSM_MsgCertCrls *pMsgCertCrls, // IN,Originator(s) certs+++
                                           //   (May be NULL).
            CSM_Buffer *&pEncodedBlob);     // OUT, Resulting SignedData/ContentInfo.
};

//////////////////////////////////////////////////////////////////////////
// The application could use CSM_DecryptData if it wants to directly
// manipulate the EnvelopedData snacc class
class CSM_DecryptData
{
private:
   CSM_Buffer* TryThisInstance(CSM_CSInst *pInst, 
         CSM_CertificateChoiceLst *pOrigCerts);

public: 
   EnvelopedData m_SnaccEnvelopedData; // SNACC 

   // CONSTRUCTORS
   CSM_DecryptData() {}
   CSM_DecryptData(CSM_Buffer *pbufEnvelopedData);
   CSM_DecryptData(CSM_Buffer *pbufEnvelopedData, CSMIME *pCSMIME);

   void PreProc(CSMIME *pCSMIME);
   void PreProc(CSM_Buffer *pbufEnvelopedData, CSMIME *pCSMIME);
   void Decode(CSM_Buffer *pbufEnvelopedData);

   // Decrypt the provided ASN.1 encoded EnvelopedData based on
   // the provided parameters
   void Decrypt(CSMIME *pCSMIME,      // IN, including instances
         CSM_CertificateChoiceLst *pOrigCerts, // IN, originator certs
         CSM_Buffer *pbufDecryptedContent); // OUT, decrypted content
};

//////////////////////////////////////////////////////////////////////////
// The application could use CSM_EncryptData if it wants to directly
// manipulate the EnvelopedData snacc class
class CSM_EncryptData
{
private:
   void LoadFromMsgCertCrls(CSM_MsgCertCrls *pMsgCertCrls);
   void AddUKMToOriginatorInfo(CSM_Buffer *pUKM, CSM_Alg *pAlg);
   CSM_Buffer* GetUKMFromOriginatorInfo(CSM_Alg *pAlg);

public: 
   EnvelopedData m_SnaccEnvelopedData; // SNACC 

   // Encrypt and encode into private member pbfEncodedBlob
   void Encrypt(CSMIME *pCSMIME,      // IN,logged-on Instance list
         CSM_MsgCertCrls *pMsgCertCrls,     // IN,Originator(s) certs
                                            // (May be NULL).
         CSM_RecipientLst *pRecipients,     // IN,Recipient Certs.
         CSM_OID *poidContentType,          // IN, content type
         CSM_Buffer *pContent,              // IN, content for env. data
         CSM_Alg *pContentEncryptionAlg,    // IN, content encryption Alg
         CSM_Buffer *pOutputBuf);           // OUT, resulting EnvelopedData/
                                            //   ContentInfo.
};

//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
// SFL HIGH-LEVEL API CLASSES for SFL OPERATIONS
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////
// CSM_Content represents the inner (or encapsulated) content.  The inner
// content is the ContentInfo->[SignedData|EnvelopedData]->ContentInfo.
class CSM_Content {
public:
   enum {
         CONTENT_DATA_INVALID, 
         CONTENT_DATA_DATA, // we decode this
         CONTENT_DATA_ANY // we do not decode this
         };

   CSM_OID m_contentType; // from ContentInfo->contentType
   CSM_Buffer m_content; // used if CONTENT_DATA_DATA or CONTENT_DATA_ANY
   long m_lType; // value from enum above

   // CONSTRUCTORS
   CSM_Content() { m_lType = CONTENT_DATA_INVALID; } // empty constructor
   CSM_Content(char *pszContent)  // construct DATA from null term string
   { SetContent(pszContent); }
   CSM_Content(CSM_Buffer *pContent)  // construct DATA from buffer
   { SetContent(pContent); }
   CSM_Content(CSM_Buffer *pContent, CSM_OID &tOID) 
   { SetContent(pContent, tOID); }
   CSM_Content(CSM_Content *pContent)  // construct from another CSM_Content
   { SetContent(pContent); }

   // Set functions
   void SetContent(char *pszContent) {
         m_content.Set(pszContent);
         m_lType = CONTENT_DATA_DATA;
      }
   void SetContent(CSM_Buffer *pContent) {
         m_content = *pContent;
         m_lType = CONTENT_DATA_DATA;
      }
   void SetContent(CSM_Buffer *pContent, CSM_OID &tOID) {
         m_content = *pContent;
         m_contentType = tOID;
         if (tOID != id_data)
           m_lType = CONTENT_DATA_ANY;
         else
           m_lType = CONTENT_DATA_DATA;
      }
   void SetContent(CSM_Content *pContent) {
         // copy pContent into m_content
         //m_lType = CONTENT_DATA_DATA;
         *this = *pContent;
      }

};

//////////////////////////////////////////////////////////////////////////
// CSM_CommonData is inherited by all high level message classes as the
// component that stores content and anything else that ends up being
// common
class CSM_CommonData
{
protected:
   short m_version;
   // m_pContentValue contains content of signedData, envelopedData, or
   // receipt, or the ASN ANY data of an unknown; or the data content
   // of an OCTET STRING (usually a mime body part)
   CSM_Content *m_pContentValue;
   // m_pEncodedBlob is the actual encoded ASN.1 binary; used for input to
   // recipient (verify, decrypt) processing and output for originator
   // (sign, encrypt) processing.
   CSM_Buffer *m_pEncodedBlob;

   void Clear() 
         { m_version = 0; m_pContentValue = NULL; m_pEncodedBlob = NULL; }

public:
   CSM_CommonData() { Clear(); }

   // attribute members
   CSM_Buffer *GetEncodedBlob() { return m_pEncodedBlob; }
   void SetEncodedBlob(CSM_Buffer *pBlob);
   void SetEncodedBlob(CSM_Content *pMsgBlob);
   CSM_Content *GetEncapContent();   // returns inner content.
   void SetEncapContent(CSM_Content &EncapContent);
   void SetEncapContent(CSM_Buffer *pEncapContent, long type);
   void setContentType(CSM_OID &oContentType);
};

//////////////////////////////////////////////////////////////////////////
// Intent of the CSM_ContentInfoMsg class is that an incomming message be 
// processed to determine the type:  for example...
//    CSM_DecryptMsg *pEnvData;
//    CSM_ContentInfoMsg *pCIM = new CSM_ContentInfoMsg(CSM_BufFile *pMsg);
//    if (pCIM->GetContentType() == EnvelopedDataOID)
//       pEnvData = new CSM_DecryptMsg(pCIM, pCSMIME);
//    else ...            // same for SignedData.
//
class CSM_ContentInfoMsg: public CSM_CommonData
{
private:
    ContentInfo *m_pSNACCContentInfo;
public:
    CSM_ContentInfoMsg();
    CSM_ContentInfoMsg(CSM_Buffer *pBuf);
    bool IsSignedData();
    bool IsEnvelopedData();
    bool IsData();
    CSM_Buffer *GetEncodedCI();
    void SetEncodedCI(CSM_Buffer &Buffer);
};
                     
//////////////////////////////////////////////////////////////////////////
//  CSM_VerifyMsg
//  This class handles the decoding of an SMIME SignedData ASN.1 encoded 
//  component.  It provides all data to the application.
//  SignedData ::=
//  SEQUENCE {
//      version                Version,
//      digestAlgorithms       DigestAlgorithmIdentifiers,
//      contentInfo            ContentInfo,
//      certificates       [0] IMPLICIT CertificateSet OPTIONAL,
//      crls               [1] IMPLICIT CertificateRevocationLists OPTIONAL,
//      signerInfos            SignerInfos }
class CSM_VerifyMsg: public CSM_CommonData,
                     private CSM_VerifyData
{
public:
   CSM_MsgCertCrls *m_pMsgCertCrls;  // handles all certs and CRLs ops for
                                     //  msg encoding/decoding.
   CSM_MsgSignerInfos *m_pSignerInfos;  // handles all signerInfo processing.

   // constructors
   CSM_VerifyMsg() { m_pMsgCertCrls=NULL; m_pSignerInfos=NULL; };
   CSM_VerifyMsg(CSM_ContentInfoMsg *pCI);   // extract content info for SD.
   // the constructor below also pre-processes the content using the
   // CSMIME/CSInst classes so it can mark applicable instances for use
   CSM_VerifyMsg(CSM_Content *pMsgBlob) 
   { m_pMsgCertCrls=NULL; m_pSignerInfos=NULL; SetEncodedBlob(pMsgBlob); }
   CSM_VerifyMsg(CSM_Buffer *pBlob) 
   { m_pMsgCertCrls=NULL; m_pSignerInfos=NULL; SetEncodedBlob(pBlob); }
   CSM_VerifyMsg(CSMIME *pCSMIME, CSM_Buffer *pBlob) 
   {
        m_pMsgCertCrls=NULL; m_pSignerInfos=NULL;
        SetEncodedBlob(pBlob); 
        PreProc(pCSMIME); 
   }
   SM_RET_VAL PreProc(CSMIME *pCSMIME, CSM_Buffer *pCSMBlob);

   // asn.1 decode and mark applicable instances
   SM_RET_VAL PreProc(CSMIME *pCSMIME);

   // do the signature verification
   SM_RET_VAL Verify(CSMIME *pCSMIME);
   SM_RET_VAL Verify(CSMIME *pCSMIME, CSM_Buffer &CSMBlob)
   {     SM_RET_VAL status=0;
         if ((status=PreProc(pCSMIME, &CSMBlob)) == 0)
           status = Verify(pCSMIME);
         return(status);   
   }

   // generate and return the signed receipt
   CSM_Buffer* GetSignedReceipt();
};

//////////////////////////////////////////////////////////////////////////
//  CSM_SignMsg
//  This class handles the creation of an SMIME SignedData ASN.1 encoded 
//  component.
//  (See comments above in CSM_VerifyMsg for the ASN.1 definition).
//  A number of elements of this message are loaded automatically by the 
//  instances specified as a parameter to SM_Sign():  
//  the DigestAlgorithms, originator certs, SignerInfos (based on sessions 
//  and attributes). 
class CSM_HashDef
{
public:
    CSM_HashDef() { m_pOID = NULL; m_pHash = NULL; }
    CSM_HashDef(CSM_OID *pOID, CSM_Buffer *pBuf) 
    { m_pOID = new CSM_OID(*pOID); m_pHash = new CSM_Buffer(*pBuf); }
    ~CSM_HashDef() 
    { 
        if (m_pOID) delete m_pOID;
        if (m_pHash) delete m_pHash;
    }
    CSM_OID *m_pOID;        // Hash encryption OID
    CSM_Buffer *m_pHash;    // Hash result for that Digest OID.
};

class CSM_SignMsg: public CSM_CommonData,
                   private CSM_SignData
{
private:
   // TRUE indicates that the encoded results will include the specified
   // originator certs.  Cert path is included only if the caller has 
   // loaded the entire set of certs using CSInst::SetCertificates
   bool m_bIncludeOrigCertsFlag;   
   bool m_IncludeContent;

   SM_RET_VAL CSM_SignMsg::SignCalculateHash(CSM_CSInst *tmpCSInst, 
      SignedData *lpSignedData, CSM_HashDefLst *&pHash, 
      CSM_Buffer *&pContent, CSM_Buffer *&pHashResult);
   SM_RET_VAL sm_PutSignerInfo(
         CSM_CSInst *pCSInst, 
         SignedData *lpSignData, 
         SignerInfo *&lpSignerInfo);

public:
   CSM_SignMsg() 
   { m_pMsgCertCrls=NULL; m_pUnauthAttrs=NULL; 
     m_pAuthAttrs=NULL;m_IncludeContent=true;}
   CSM_SignMsg(CSM_Content  *pMsgBlob) {
     m_pMsgCertCrls=NULL; m_pUnauthAttrs=NULL; 
     m_pAuthAttrs=NULL;m_IncludeContent=true;
     if (pMsgBlob)
       SetEncapContent(*pMsgBlob); }
   CSM_SignMsg(CSM_Buffer *pBlob) {
     m_pMsgCertCrls=NULL; m_pUnauthAttrs=NULL; 
     m_pAuthAttrs=NULL;m_IncludeContent=true;
     SetEncapContent(pBlob, CSM_Content::CONTENT_DATA_DATA); }

   CSM_MsgCertCrls *m_pMsgCertCrls;  // handles all certs and CRLs ops for
                                     //  msg encoding/decoding.
   CSM_MsgAttributes *m_pUnauthAttrs;// message attributes, most are common
   CSM_MsgAttributes *m_pAuthAttrs;  // message attributes, most are common
                                     //  to all CSInstances.(RWC; TBD;DAVE???)

   SM_RET_VAL Sign(CSMIME *pCSMIME);

   void SetIncludeContentFlag(bool bFlag) { m_IncludeContent = bFlag; }
   void SetIncludeOrigCertsFlag(bool bFlag) { 
         m_bIncludeOrigCertsFlag = bFlag; }

};

//////////////////////////////////////////////////////////////////////////
//  CSM_AddSignMsg
//  This class handles the addition of a signature to an SMIME SignedData 
//  ASN.1 encoded component.
//  (See comments above in CSM_VerifyMsg for the ASN.1 definition).
//  A number of elements of this message are loaded automatically by the 
//  instances
//  specified as a parameter to SM_Sign():  the DigestAlgorithms, originator
//  certs, SignerInfos (based on sessions and attributes). 
//  This particular class contains elements of both the sign and verify 
//  operations due to the odd nature of the update of an existing msg.
//  In particular, pay attention to the Counter Signature comments, where
//  the original message SignerInfos must be navigated and signed in a 
//  particular manner.  
//  FOR COUNTER SIGNATURE:  It is expected that the user will navigate the 
//  SignerInfo list to locate the desired SI to counter sign, pass this SI 
//  class pointer to the CSM_CounterSignature->generateCounterSignature() 
//  function and CSM_CounterSignature->getCounterSignature() for the 
//  attribute generation.
class CSM_AddSignMsg: public CSM_CommonData,
                      public CSM_VerifyData
{
public:
   CSM_AddSignMsg() {};
   CSM_AddSignMsg(CSM_Content *pMsgBlob) { SetEncodedBlob(pMsgBlob); }
   CSM_AddSignMsg(CSM_Buffer *pBlob) { SetEncodedBlob(pBlob); }
   CSM_AddSignMsg(CSMIME *pCSMIME, CSM_Buffer *pBlob) {
         SetEncodedBlob(pBlob); 
         PreProc(pCSMIME); }
};

//////////////////////////////////////////////////////////////////////////
// The application would use CSM_VerifyReceiptMsg after determining
// via CSM_ContentInfoMsg that it has a ContentInfo->SignedData->Receipt
class CSM_VerifyReceiptMsg: private CSM_VerifyData
{
public:
   // Set original message for SM_ValReceipt() processing.
   void SetOriginalMessage(CSM_Buffer *pOrigMsg);
   CSM_CommonData *GetCommonData() { return (CSM_CommonData *)this; }
};

//////////////////////////////////////////////////////////////////////////
// The application would use CSM_DecryptMsg to decrypt the content in
// a ContentInfo/EnvelopedData.
class CSM_DecryptMsg : public CSM_CommonData,
                       private CSM_DecryptData
{
private:
   void Clear() {
      m_pRecipients = NULL;
      m_pOriginatorInfo = NULL;
   }

   void AddSNACCRecipients(RecipientInfos *pRILst);

public: 
   CSM_DecryptMsg() { Clear(); }
   CSM_DecryptMsg(CSM_Buffer *pBlob) : CSM_DecryptData(pBlob)
      { Clear(); if (pBlob) SetEncodedBlob(pBlob); }
   CSM_DecryptMsg(CSM_Buffer *pBlob, CSMIME *pCSMIME);
   // TBD, construct from CSM_ContentInfoMsg or CSM_CommonData

   // Encapsulates the RecipientInfos from the EnvelopedData
   CSM_RecipientLst *m_pRecipients;

   // Encapsulates the OriginatorInfo from the EnvelopedData
   CSM_OriginatorInfo *m_pOriginatorInfo;

   // operational member
   void Decrypt(CSMIME *pCSMIME);
   void PreProc(CSMIME *pCSMIME);
   void PreProc(CSM_Buffer *pBlob, CSMIME *pCSMIME);
};

//////////////////////////////////////////////////////////////////////////
//  CSM_EncryptMsg
class CSM_EncryptMsg : public CSM_CommonData,
                       private CSM_EncryptData
{
private:
    CSM_OID *m_poidContentEncrypt; // Specified OID for conent encryption.
    bool m_bIncludeOrigCertsFlag;   

    void Clear() {
       m_bIncludeOrigCertsFlag = false;
       m_pMsgCrtCrls = NULL;
       m_pRecipients = NULL;
       m_poidContentEncrypt == NULL;
    }

public: 
   CSM_MsgCertCrls *m_pMsgCrtCrls;
   CSM_RecipientLst *m_pRecipients;

   CSM_EncryptMsg();
   CSM_EncryptMsg(CSM_Buffer *pBlob);
   // TBD, necessary? got rid of CSM_ContentInfoMsg constructor
   //CSM_EncryptMsg(CSM_ContentInfoMsg *pCI);
   CSM_EncryptMsg(CSM_Content  *pContent);
   ~CSM_EncryptMsg();

   // operational member
   void Encrypt(CSMIME *pCSMIME);

   void SetIncludeOrigCertsFlag(bool bFlag) { 
         m_bIncludeOrigCertsFlag = bFlag; }

   void SetContentEncryptOID(CSM_OID *pOID) 
         { m_poidContentEncrypt = new CSM_OID(*pOID); }
};



//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
// SFL LOW-LEVEL MESSAGE COMPONENT CLASSES
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////
// CSM_MsgAttributes represents the entire collection of either
// authenticated or unauthenticated attributes in a SignedData
class CSM_MsgAttributes 
{
private:
   CSM_Buffer     *m_pEncodedAttrs;
   AuthAttributes *m_pSNACCAuthAttributes;
   UnauthAttributes *m_pSNACCUnauthAttributes;
   SM_RET_VAL LoadSNACCAuthAttrs(){return(0);};
   void UnLoadSNACCAuthAttrs(){};
   SM_RET_VAL LoadSNACCUnauthAttrs(){return (0); };
   void UnLoadSNACCUnauthAttrs(){};
public:
   // Constructors
   CSM_MsgAttributes()
   {  m_pAttrs=NULL; m_pSNACCAuthAttributes=NULL;
      m_pSNACCUnauthAttributes=NULL; m_pEncodedAttrs=NULL;  }

   CSM_AttribLst  *m_pAttrs;

   // Add individual attributes.
   void GetMessageDigest(CSM_Buffer *&pMessageDigest);
   void GetSigningTime(char *&pSigningTime);
   void GetCounterSignature(CSM_MsgSignerInfo *&pCounterSignature);
   void GetReceiptRequest(CSM_ReceiptRequest *&pReceiptRequest);
   void GetContentHints(long &lDirStringType, char *&lpszDirString, 
         bool &bReceipt);
   void GetSecurityLabel(CSM_SecLbl *&pSecurityLabel);
   void GetMailList(CSM_MlLst *&pMailList);
   void GetGeneralAsn(CSM_GeneralAsn *&pGeneralAsn);
   // The following set of functions will load/unload the appropriate SNACC
   //  data structure as well as encode/decode the ASN.1 buffer as necessary.
   AuthAttributes *GetSNACCAuthAttributes() 
   { if (m_pSNACCAuthAttributes == NULL)
        LoadSNACCAuthAttrs();
     return m_pSNACCAuthAttributes; 
   }
   void SetSNACCAuthAttributes(AuthAttributes *pSNACCAuthAttributes)
   {
        if (m_pSNACCAuthAttributes)
            delete m_pSNACCAuthAttributes;
        m_pSNACCAuthAttributes = pSNACCAuthAttributes;
        if (m_pEncodedAttrs)
        {
            delete m_pEncodedAttrs;
            m_pEncodedAttrs = NULL;
        }
        UnLoadSNACCAuthAttrs();
   }
   CSM_Buffer *GetAuthEncodedAttrs() 
   { 
        if (m_pEncodedAttrs == NULL)
        {
            if (GetSNACCAuthAttributes() != NULL)
              ENCODE_BUF(m_pSNACCAuthAttributes, m_pEncodedAttrs);
        }
        return(m_pEncodedAttrs);
   }
   void SetAuthEncodedAttrs(AuthAttributes *pSNACCAuthAttributes) 
   { 
        if (m_pEncodedAttrs)
            delete m_pEncodedAttrs;
        if (m_pSNACCAuthAttributes)
            delete m_pSNACCAuthAttributes;
        DECODE_BUF(m_pSNACCAuthAttributes, m_pEncodedAttrs);
        UnLoadSNACCAuthAttrs();
   }
};

//////////////////////////////////////////////////////////////////////////
// This class encapsulates the SignerInfos from SignedData
// This class is only used for verification, not production of SignedData.
class CSM_MsgSignerInfo
{
private:
   SignerInfo *m_pSignerInfo;   // from the SignerInfos above.
   CSM_BufferLst *m_pCerts;      // certs for public key of SignerInfo
                        //  for verification (May be loaded by
                        //   incomming SignedData message or application for
                        //   processing convenience).
   bool m_Verified;     // indicates that this SignerInfo has been
                        //   signature verified.

public:
   CSM_MsgSignerInfo(SignerInfo *pSignerInfo)
   { m_Verified=false; m_pSignerInfo = pSignerInfo; 
     m_pCerts=NULL; m_Verified=false; 
     m_pAuthAttrs=NULL; m_pUnauthAttrs=NULL; }

   SignerInfo* GetSignerInfo() { return m_pSignerInfo; }

   CSM_MsgAttributes *m_pUnauthAttrs;// message attributes, most are common
   CSM_MsgAttributes *m_pAuthAttrs;  // message attributes, most are common
                                     //  to all CSInstances.(RWC; TBD;DAVE???)
   // The following member functions will extract the specified component 
   //  from the present SignerInfo (as set by the above "set...Info()" calls).
   CSM_IssuerAndSerialNumber* GetIssuerAndSerial();
   CSM_Alg* GetDigestId();
   CSM_Alg* GetSignatureId();

   // load certs to be used for signature verification
   void SetSignerInfoCerts(CSM_BufferLst *pCerts);
   bool IsVerified(){ return m_Verified; }
   void SetVerified(bool bVerified){ m_Verified = bVerified; }
};

//////////////////////////////////////////////////////////////////////////
class CSM_MsgSignerInfos: public CSM_MsgSignerInfoLst
{
private:
   // designates current SignerInfo with 
   //  "flagApplicable" flag set by PreProc; this 
   //  implies that this SignerInfo can be processed
   //  by the PreProc sessions.
   SignerInfos *m_pNextApplicableSignerInfo;

public:
    CSM_MsgSignerInfos(){ m_pNextApplicableSignerInfo=NULL; }
    CSM_MsgSignerInfos(SignerInfo *pSNACCSignerInfo)
    { m_pNextApplicableSignerInfo=NULL;
      Set(new CSM_MsgSignerInfo(pSNACCSignerInfo)); }
    CSM_MsgSignerInfos(CSM_MsgSignerInfo *pCSMSignerInfo)
    { m_pNextApplicableSignerInfo=NULL;
      Set(pCSMSignerInfo); }
    ~CSM_MsgSignerInfos();

    // Get next SM_Signer_info that is valid for the specified list of sessions.
    CSM_MsgSignerInfo *GetNextApplicableSignerInfo(CSMIME *pCSInsts);
    CSM_MsgSignerInfo *GetFirstApplicableSignerInfo(CSMIME *pCSInsts);
};

/**************************************************************************/
/* The content type of the encapsulated content in the message - may      */
/* provide content type as an integer value (built_in) or as an OID       */
/* (external).                                                            */
/**************************************************************************/
class CSM_CntType
{
public:
   long m_BuiltIn; /* content type */
   CSM_OID m_External; /* object identifier (OID) (not ASN! encoded) */
   long m_Subtype; /* ContentType subtype */
};

/**************************************************************************/
/*  Handle SignerInfo authenticated and unauthenticated Attributes.       */
/**************************************************************************/
/**************************************************************************/
/* Structure for signed receipt requested                                 */
/**************************************************************************/
class CSM_ReceiptRequest
{
public:
   CSM_CntType *m_encapsulated_content_type; /* optional */
   CSM_Buffer m_SignedContentIdentifier;
   long m_lReceiptInd;      /* Indication of signed receipts */
   CSM_DNLst m_ReceiptsFrom;
   CSM_DNLst *m_ReceiptsTo; /* optional */

   CSM_ReceiptRequest() { };

   //This function loads from either ReceiptsFrom or ReceiptsTo
   void Load_GeneralName(CSM_DN *pDNS,GeneralName *&pGenName)
   {
     if(pGenName == NULL)
        pGenName = new GeneralName;

     pGenName->choiceId = GeneralName::directoryNameCid;
     pGenName->directoryName = new Name;
      *pGenName->directoryName = *(Name *)*pDNS;

   }
};

/**************************************************************************/
/*  Security Categories structure                                         */
/**************************************************************************/
class CSM_SecCat
{
public:
   CSM_OID m_Type;
   //Value is expected to be ASN.1 Encoded
   CSM_Buffer m_Value;
}; 

/**************************************************************************/
/*Structure def for Security Label none of this data is ASN1 encoded      */
/**************************************************************************/
class CSM_SecLbl
{
public:
   CSM_OID *m_PolicyId;     /* Security Policy Identifier */
   long m_lSecClass;        /* Security Classification */
   CSM_Buffer *m_pPmark;    /* Privacy Mark */
   CSM_SecCatLst *m_pSecCats;/* Security Categories */
};

//////////////////////////////////////////////////////////////////////////
class CSM_MlRecPol
{
public:
   long m_lRecPolicyType;           /* indicates receipt policy */
   CSM_BufferLst *m_pOrnameLst;     /* ORNames list */
};

//////////////////////////////////////////////////////////////////////////
class CSM_GeneralAsn
{
public:
   CSM_OID m_AsnOid;       /** OID for Any in strList. **/
   CSM_BufferLst m_StrLst;

   CSM_GeneralAsn() { };
};

/**************************************************************************/
/* Entity Identifier structure                                            */
/**************************************************************************/
class CSM_Entity
{
public:
   enum {
        ENTITY_ISSUER_SERIAL,
        ENTITY_KEY_ID };
   long m_lIdType;
   union {
      CSM_IssuerAndSerialNumber *m_pIssuerSerial;
      CSM_Buffer *m_pKeyId;
   };

   CSM_Entity() { m_lIdType = 0; m_pIssuerSerial = NULL; }
   CSM_Entity(EntityIdentifier &ei);
};

//////////////////////////////////////////////////////////////////////////
class CSM_ContentHints
{
public:
   enum {
      HINT_teletexString,
      HINT_printableString,
      HINT_bmpString,
      HINT_universalString };
   long m_lsDirStringType; // union member selected for contentDescription
   CSM_Buffer m_ContentDescription;
   CSM_OID m_OID;          // OID describing the description.
};

/**************************************************************************/
/* Linked list of ml_hst structure for each ML that has been expanded     */
/* during the message distribution.                                       */
/**************************************************************************/
class CSM_Ml
{
public:
   CSM_Entity             m_Entity;
   CSM_Buffer             m_ExpTime;          /* time of expansion of the ML */
   CSM_MlRecPol           m_MlRecPolicy;      /* indicates receipt policy */

   CSM_Ml() { };
};

//////////////////////////////////////////////////////////////////////////
class CSM_CounterSignature
{
private:
    CSM_MsgSignerInfo *m_pSignerInfo;

public:
    SM_RET_VAL GenerateCounterSignature(CSM_CSInst *pCSInst, 
         CSM_MsgSignerInfo *pSignerInfo);

    CSM_MsgSignerInfo *GetCounterSignature()
    { return m_pSignerInfo;  }

};


//////////////////////////////////////////////////////////////////////////
class CSM_Attrib
{
public:
   CSM_Attrib(){};
   // The following constructor handles MessageDigest.
   CSM_Attrib(CSM_Buffer *pMessageDigest);
   CSM_Attrib(char *pSigningTime);
   CSM_Attrib(CSM_CounterSignature *pCounterSignature);
   CSM_Attrib(CSM_ReceiptRequest *pReceiptRequest);
   CSM_Attrib(CSM_ContentHints *pContentHints);
   CSM_Attrib(CSM_SecLbl *pSecLbl);
   CSM_Attrib(CSM_MlLst *pMlLst);
   CSM_Attrib(CSM_GeneralAsn *pGeneralAsn);

   enum {
       ATTR_Message_Digest,
       ATTR_Signing_Time,
       ATTR_Counter_Signature,
       ATTR_Receipt_Request,
       ATTR_Content_Hints,
       ATTR_Security_Label,
       ATTR_MlExpHist,
       ATTR_General_ASN };

   /** RWC;OID content_type; CONSTANT, handled by lolevel functions**/
   long m_lContentType;
   union {
      CSM_Buffer *m_pMessageDigest;
      char *m_pSigningTime;
      CSM_CounterSignature *m_pCounterSignature;
      CSM_ReceiptRequest *m_pReceiptRequest;
      CSM_ContentHints *m_pContentHints;
      CSM_SecLbl *m_pSecurityLabel;
      CSM_MlLst *m_pMlExpHist;
      CSM_GeneralAsn *m_pGeneralAsn;  /**OID & ASN encoded data list. **/
    };

   // optional CSInstance specification for Sign()
   char *m_pszCSInst;   // IDs the instance's SignerInfo to contain this attr;
                        //   MAY BE NULL to indicate all SignerInfos.
   CSM_OID    *m_poid;
   CSM_Buffer *m_pEncodedAttrib;

};

//////////////////////////////////////////////////////////////////////////
// CSM_CertificateChoice class encapsulates a certificate
// CHOICE of certificate, extendedCertificate or attrCert
class CSM_CertificateChoice
{
private:
   // these members are loaded after decoding the included cert
   CSM_OID *m_pKeyOID;        // Subject Public Key Info OID
   CSM_DN *m_pSubject;        // subject dn of recipient
   CSM_DN *m_pIssuer;         // issuer dn of recipient
   CSM_Buffer *m_pCert;       // ASN.1 encoded certificate

   // these are the decoded certs, only one will be filled at any
   // given time
   Certificate *m_pSnaccCert; // ASN.1 decoded certificate
   AttributeCertificate *m_pSnaccAttr; // ASN.1 decoded attrib cert

   void Clear(); // clear members
   void Decode(); // decode m_pCert into m_pSnaccCert

public:
   enum {
        CERT_TYPE_CERT,
        CERT_TYPE_ATTR_CERT,
        CERT_TYPE_EXTENDED_CERT }; // Extended NOT SUPPORTED FOR NOW!!!
   long m_lCertType; // Indicates the component that is present in the Union

   CSM_CertificateChoice() { Clear(); }
   CSM_CertificateChoice(CSM_Buffer *pCert);
   CSM_CertificateChoice(CSM_Buffer *pCert, long lType);
   CSM_CertificateChoice(CertificateChoices &SNACCCertChoices);
   CSM_CertificateChoice(CSM_CertificateChoice &certChoice);
   ~CSM_CertificateChoice();

   CSM_Buffer* GetEncodedCert() { return m_pCert; }
   void SetEncodedCert(CSM_Buffer *pCert);
   void SetEncodedCert(CSM_Buffer *pCert, long lType);
   void SetSNACCCertChoices(CertificateChoices &SNACCCertChoices);

   CSM_OID* GetKeyOID(); // return the public key OID
   CSM_Buffer* GetPublicKey(); // return public value from cert
   CSM_DN* GetIssuer(); // return issuer of cert
   CSM_DN* GetSubject(); // return subject of cert
   CSM_Buffer* GetSerial(); // return serial number of cert
   CSM_Alg* GetPublicKeyAlg(); // return subject public key alg
};

//////////////////////////////////////////////////////////////////////////
// CSM_Recipient represents a single recipient for encrypting and 
// decrypting EnvelopedDatas.
class CSM_Recipient : public CSM_CertificateChoice
{
private:
   bool m_bDecrypted;
   // the three members below correspond to the SNACC RecipientInfo stuff
   CSM_IssuerAndSerialNumber *m_pRID; // TBD, union w/ RecipKeyID and mlKeyID
   CSM_Entity *m_pOriginatorCert;
   CSM_Alg *m_pKeyEncryptionAlg;

   void Clear() ;

public:
   CSM_Recipient() { Clear(); }
   CSM_Recipient(CSM_Buffer *pCert) : CSM_CertificateChoice(pCert)
         { Clear(); }
   CSM_Recipient(CSM_Buffer *pCert, long lType) : 
         CSM_CertificateChoice(pCert, lType)
         { Clear(); }

   CSM_IssuerAndSerialNumber* GetIssuerAndSerial() { return m_pRID; }
   CSM_Entity* GetOriginatorEntity() { return m_pOriginatorCert; }
   CSM_Alg* GetKeyEncryptionAlg() { return m_pKeyEncryptionAlg; }

   void SetFromSNACCRecipientInfo(RecipientInfo *pRI);

   bool WasDecrypted() { return m_bDecrypted; }
};

//////////////////////////////////////////////////////////////////////////
class CSM_Alg
{
private:
   AlgorithmIdentifier *m_pSNACCAlgId;

   void Clear();

public:
   // public member variables
   CSM_OID     *m_pId;             // always required
   CSM_Buffer  *m_pParams;         // optional

   // CONSTRUCTORS
   CSM_Alg();
   CSM_Alg(CSM_OID &AlgOid);
   CSM_Alg(AlgorithmIdentifier &SNACCAlgId);

   // OPERATIONAL methods
   CSM_OID *GetId();
   CSM_Buffer *GetParams() { return m_pParams; }
   AlgorithmIdentifier *GetSNACCAlgId();
   void PutSNACCAlg(AlgorithmIdentifier *pSNACC_Alg);

   // OPERATORS
   bool operator == (CSM_Alg &AlgId);
   bool operator != (CSM_Alg &AlgId);
#if !(defined(SUNOS) && defined(GNUG272))
   bool operator == (const AsnOid &snaccOid) {
      if (m_pId == NULL) return false;
      return (*m_pId == snaccOid); }
#endif
   bool operator != (const AsnOid &snaccOid) {
      if (m_pId == NULL) return false;
      return !(*m_pId == snaccOid); }
};

//////////////////////////////////////////////////////////////////////////
class CSM_UserKeyMaterial
{
private:
   void Clear() { m_pAlgorithm = NULL; m_pUKM = NULL; }

public:
   CSM_UserKeyMaterial() { Clear(); }
   CSM_UserKeyMaterial(CSM_Alg *pAlg, CSM_Buffer *pUKM)
      { m_pAlgorithm = pAlg; m_pUKM = pUKM; }

   CSM_Alg *m_pAlgorithm;
   CSM_Buffer *m_pUKM; 
};

//////////////////////////////////////////////////////////////////////////
class CSM_MsgCertCrls
{
private:
   CSM_CertificateChoiceLst *m_pCerts; // certificates from PreProc or for
                                       //  sign/encrypt operations.
   CSM_CertificateChoiceLst *m_pACs;
                              // AttributeCertificates from PreProc or for
                              //  sign/encrypt operations.
   CSM_BufferLst *m_pCRLs;    // crls from PreProc or for
                              //  sign/encrypt operations.
   void Clear() { m_pCerts=NULL; m_pACs=NULL; m_pCRLs=NULL; }

public:
   // Gets return pointers to the data in this class
   CSM_CertificateChoiceLst *GetCertificates() { return m_pCerts; }
   CSM_BufferLst *GetCRLs() { return m_pCRLs; }
   CSM_CertificateChoiceLst *GetACs(){ return m_pACs; }

   // SetCertificates from a CSM_BufferLst assumes that the
   // provided certs are regular certs and copies them into
   // m_pCerts
   void SetCertificates(CSM_BufferLst *pCerts);

   // SetCertificates from a CSM_CertificateChoiceLst
   // copies the regular certs into m_pCerts and the
   // ACs into m_pACs
   void SetCertificates(CSM_CertificateChoiceLst *pCerts);

   // SetCRLs copies the provided CRLs into m_pCRLs
   void SetCRLs(CSM_BufferLst *pCrls);

   // AddCert adds a single cert to the appropriate location
   // note, pointer is copied so caller should not free ptr after
   // passing it to this function
   void AddCert(CSM_CertificateChoice *pCert);

   // use the Deletes to clear the appropriate components of this class
   void ClearCerts();
   void ClearACs();
   void ClearCRLs();

   // PutSNACCCerts decodes the certs in this class and loads
   // their decoded values into the provided (or allocated)
   // CertificateSet
   void PutSNACCCerts(CertificateSet *&pCertificateSet);
   // SetSNACCCerts places the provided snacc decoded certs
   // into this appropriately
   void SetSNACCCerts(CertificateSet *pCertificateSet);
   // SetSNACCCrls places the provided snacc decoded crls
   // into this appropriately
   void SetSNACCCrls(CertificateRevocationLists *pCrls);

   // constructors and destructor
   CSM_MsgCertCrls() { Clear(); }
   CSM_MsgCertCrls(CSM_CertificateChoiceLst *m_pCerts)
         { Clear(); SetCertificates(m_pCerts); }
   CSM_MsgCertCrls(CSM_BufferLst *m_pCerts)
         { Clear(); SetCertificates(m_pCerts); }
   CSM_MsgCertCrls(CertificateSet *pCertificateSet)
         { Clear(); SetSNACCCerts(pCertificateSet); }
   ~CSM_MsgCertCrls();
};

//////////////////////////////////////////////////////////////////////////
class CSM_OriginatorInfo : private CSM_MsgCertCrls
{
private:
   void Clear() { m_pMsgCertCrls = NULL; m_pUKMs = NULL; }

public:
   CSM_MsgCertCrls *m_pMsgCertCrls;
   CSM_UserKeyMaterials *m_pUKMs;

   CSM_OriginatorInfo() { Clear(); }
   CSM_OriginatorInfo(CSM_BufferLst *pCerts);
   CSM_OriginatorInfo(CSM_CertificateChoiceLst *pCerts);
   CSM_OriginatorInfo(CertificateSet *pSnaccCertSet);
   CSM_OriginatorInfo(OriginatorInfo *pSnaccOI);

   // AddSNACCOrigInfo adds the provided SNACC OriginatorInfo to
   // this class
   void AddSNACCOrigInfo(OriginatorInfo *pOI);
   // GetOriginatorDns returns the DNs of each cert in this class
   CSM_DNLst* GetOriginatorDns();
   // SetSNACCUKMs adds the provided UserKeyingMaterials to m_pUKMs
   void SetSNACCUKMs(UserKeyingMaterials *pUKMs);
};

//////////////////////////////////////////////////////////////////////////
// NOTE:: Parameters are not necessary for these algorithms:  Fortezza uses
//  the parameters from the Certpath, RSA and DH are included in the 
//  certificate.  If the parameters are necessary in the future, it can be
//  added to the MAB data structures.
//
// MAB_Entrydef represents a single entry in the address book
class MAB_Entrydef 
{
private:
   // SNACC decoded cert, retained if decoded once.
   Certificate *m_pCertificate;
   // issuer and serial number (for easy ID).
   CSM_IssuerAndSerialNumber *m_pIssuer;  

public:
   MAB_Entrydef(){ memset(this, '\0', sizeof(MAB_Entrydef)); }
   ~MAB_Entrydef();

   char *m_pszAlias; // alias name, main index to data.

   // X.500 DN used as the subject DN on this particular entry's 
   // certificate.  The format of this dn string is
   // c=US@o=US Governement etc...
   char *m_pszCertificateDN; 

   // relative path identifying the file containing the cert binary
   // MAB_AB_def.m_szGlobalPath is prepended to this file name
   CSM_Buffer *m_pCertFile;

   // information containing the private key for the cert if applicable
   // the m_pPrivateOID helps the CTI determine how to interpret this
   // information whether it be a slot number on a fortezza card or
   // a file name containing a PKCS#8 EncryptedPrivateKeyInfo
   CSM_Buffer *m_pPrivateInfo;

   // identifies the type of data stored in m_pPrivateInfo
   CSM_OID *m_pPrivateOID;

   // GetCertificate forces decoding of cert on first call,
   // otheriwse returns pointer to already decoded cert
   Certificate *GetCertificate();

   // GetIssuer decodes and extracts the issuer serial number
   // on the first call, otherwise it uses what is available
   CSM_IssuerAndSerialNumber *GetIssuer();
};

//////////////////////////////////////////////////////////////////////////
// MAB_AB_def represents the address book
class MAB_AB_def
{
public:
   enum { // types used with FindRecord
      MABTYPE_DN,
      MABTYPE_ALIAS
   };

   CSM_List<MAB_Entrydef> *m_pEntries; // list of entries in address book
   char m_szGlobalPath[256]; // global path to files of AB Entry.

   // constructors and destructors
   MAB_AB_def(); // create empty mab
   MAB_AB_def(char *pszFileName); // create mab from file
   ~MAB_AB_def();
   
   // Init reads the mab file into this class
   SM_RET_VAL Init(char *pszFileName);
   // returns the cert for the specified DN
   SM_RET_VAL FindCertDN(char *pszDN, CSM_Buffer *result);
   // returns the specified record (MAB_Entrydef class)
   MAB_Entrydef *FindRecord(char *pszID, long lType);
   // returns the cert with the specified alias
   SM_RET_VAL FindCertAlias(char *pszAlias, CSM_Buffer *result);
   // returns the cert with the specified issuer and serial number
   SM_RET_VAL FindIssuer(CSM_IssuerAndSerialNumber &csmIssuer, 
            CSM_Buffer *result);
   SM_RET_VAL FillCertPath(CSM_CertificateChoice *pCertToPath,
      CSM_MsgCertCrls *pMsgCertCrls);
};

#endif // _SM_API_H_
// EOF sm_api.h
